/*Maysa Macedo- maysa ( at) ime.usp.br*/

/*classe que acessa os métodos remotos e possui alguns atributos que controlam a partida.*/

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.net.MalformedURLException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;


public class ControladorPartidaCliente {

    private static Map cacheObjetos = new HashMap();
    private static CenarioTO cenario;
    private static JogadorTO jogador;
    private static PrimordialSoupMain telaPrincipal;
    private Boolean segundaAdicaoAmeba;
    private Integer controladorEtapa;

    /**
     * @return the telaPrincipal
     */
    public static PrimordialSoupMain getTelaPrincipal() {
        return telaPrincipal;
    }
    

    private static ControladorPartidaCliente instance;

    private ControladorPartidaCliente(){
       segundaAdicaoAmeba=false;
       controladorEtapa = 0;
       
    }

    public static ControladorPartidaCliente getInstance(){
        if(instance == null){
            instance = new ControladorPartidaCliente();
        }
        return instance;
    }

    /**
     * Retorna um stub(proxy) de um objeto remoto. Os stubs serao guardados em um cache
     * @param nome
     * @return
     * @throws RemoteException
     */
    public Object getObjetoRemoto(String nome) throws RemoteException{
        try{
            if(!cacheObjetos.containsKey(nome)){
                cacheObjetos.put(nome, Naming.lookup("rmi://localhost:1099/" + nome));
            }
        }catch( MalformedURLException e ) {
            System.out.println();
            System.out.println( "MalformedURLException: " + e.toString() );
        }
        catch( RemoteException e ) {
            System.out.println();
            System.out.println( "RemoteException: " + e.toString() );
        }
        catch( NotBoundException e ) {
            System.out.println();
            System.out.println( "NotBoundException: " + e.toString() );
        }
        catch( Exception e ) {
            System.out.println();
            System.out.println( "Exception: " + e.toString());
        }
        return cacheObjetos.get(nome);
    }


    public CenarioTO logar(JogadorTO jogadorto) throws RemoteException{
        ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");

        cenario = stub.logar(jogadorto);
        return cenario;
    }


     public void atualizarCenario() throws RemoteException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
           CenarioTO cenarioBackup = null;
           ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
           cenarioBackup = stub.atualizarCenario(cenario.getCod_partida(), cenario.getAtualizacaoCenario());
           System.out.println("Carregamento automático.");
           if (cenarioBackup != null){
                cenario = cenarioBackup;
           }
           carregarCenario();
           System.out.println("Fim do carregamento automático");
           
     }

    

      public void setCenario(CenarioTO cenarioTo)throws RemoteException{
         cenario = cenarioTo;
     }
     public CenarioTO getCenario()throws RemoteException{
         return cenario;
     }
     public void setJogador(JogadorTO jogadorTo)throws RemoteException{
         jogador = jogadorTo;
     }

      public JogadorTO getJogador() throws RemoteException{
        return jogador;
    }
/**
     * @return the escolherMarcador
     */
    public Boolean getSegundaAdicaoAmeba() {
        return this.segundaAdicaoAmeba;
    }

    /**
     * @param escolherMarcador the escolherMarcador to set
     */
    public void setSegundaAdicaoAmeba(Boolean segundaAdicaoAmeba) {
        this.segundaAdicaoAmeba = segundaAdicaoAmeba;
    }

    /**
     * @return the escolheuAmeba
     */
    public Integer getcontroladorEtapa() {
        return controladorEtapa;
    }

    /**
     * @param escolheuAmeba the escolheuAmeba to set
     */
    public void setcontroladorEtapa(Integer controladorEtapa) {
        this.controladorEtapa = controladorEtapa;
    }



     public void setTelaPrincipal(PrimordialSoupMain telaprincipal)throws RemoteException{
         telaPrincipal = telaprincipal;
     }

     public void carregarCenario()throws RemoteException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
        getTelaPrincipal().mostrarTabuleiro();

     }

      public Integer jogarDado()throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         return stub.jogar2Dado();

     }
     public Integer jogarDadoFase1()throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         return stub.jogar1Dado();

     }


      public void enviarDado(Integer valor)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         this.segundaAdicaoAmeba=true;
         cenario =   stub.enviarDado(cenario.getCod_partida(), jogador.getCodigo(), valor);
        
     }

     public void escolherMarcador(Integer cor, Integer posicao)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         this.segundaAdicaoAmeba=true;
         cenario =  stub.escolherMarcador(cenario.getCod_partida(), jogador.getCodigo(), cor, posicao);
        
     }


      public Integer[][] escolherEspaco(Integer codPartida, Integer codJogador)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         return stub.escolherEspaco(codPartida, codJogador);
     }

      public void adicionarAmeba(Integer codPartida, Integer codJogador, AmebaTO ameba, Integer x, Integer y)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.adicionarAmeba(codPartida, codJogador, ameba, x, y);
         
     }

      public void deslizarAmeba(Integer codPartida, Integer codJogador, AmebaTO ameba, Integer x1, Integer y1, Integer x2, Integer y2)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.deslizarAmeba(codPartida, codJogador, ameba, x1, y1, x2, y2);

     }

     public void movimentarAmeba(Integer codPartida, Integer codJogador, AmebaTO ameba, Integer x1, Integer y1, Integer[] xy2)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.movimentarAmeba(codPartida, codJogador, ameba, x1, y1, xy2[0], xy2[1]);

     }
     

      public Integer[][] escolherEspaco(Integer codPartida, Integer codJogador,Integer cor)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         return stub.escolherEspaco(codPartida, codJogador, cor);
     }

      public void comerNutriente(Integer codPartida, Integer codJogador,AmebaTO ameba , Integer x, Integer y, EspacoTO espaco)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.comerNutriente(codPartida, codJogador, ameba, x, y, espaco);
          

     }

    public ArrayList<HistoricoTO> carregarHistorico()throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         System.out.println("Solicitando o historico do cod_jogador:" + jogador.getCodigo());
         return stub.requisitarHistorico(jogador.getCodigo());
     }


      public void mudarParaFase2(Integer codPartida, Integer codJogador)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.mudarParaFase2(codPartida, codJogador);

     }
     public Integer calcularMulta(Integer codPartida, Integer codJogador)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         Integer multa = stub.calcularMulta(codPartida, codJogador);
         return multa;
     }

     public void pagarMulta(Integer codPartida, Integer codJogador, ArrayList<CartaGeneTO> cartasGene, Integer pontoBiologico)throws RemoteException{
         ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
         cenario = stub.pagarMulta(codPartida, codJogador, cartasGene, pontoBiologico);

     }

     public void mudarParaFase3(Integer codPartida, Integer codJogador) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.mudarParaFase3(codPartida, codJogador);

     }

     public void comprarGene(Integer codPartida, Integer codJogador,ArrayList<CartaGeneTO >cartasGene) throws RemoteException{
        ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
        cenario = stub.comprarGene(codPartida, codJogador, cartasGene);

     }
     public void mudarParaFase4(Integer codPartida, Integer codJogador) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.mudarParaFase4(codPartida, codJogador);

     }

      public void mudarParaFase5(Integer codPartida, Integer codJogador) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.mudarParaFase5(codPartida, codJogador);

     }

     public void calcularPontuacao(Integer codPartida, Integer codJogador) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.calcularPontuacao(codPartida, codJogador);

     }

     public VencedorTO carregarVencedores() throws RemoteException{
          VencedorTO vencedores = new VencedorTO();
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          vencedores = stub.requisitarVencedores();
          return vencedores;

     }

      public void interromperPartida(Integer codPartida, Integer codJogador) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.interromperPartida(codPartida, codJogador);

     }
      public void reiniciarPartida(Integer codPartida) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          cenario = stub.reiniciarPartida(codPartida);

     }
      public void expirarPartida() throws RemoteException{
        telaPrincipal.expirarPartida();
    }

     public void fecharPartida(Integer codPartida) throws RemoteException{
          ControladorRemote stub = (ControladorRemote) getObjetoRemoto("ControladorPartida");
          stub.fecharPartida(codPartida);

     }
   
    }
